-- TitanVPCap.lua
--

--[[
TODO:


]]

-- Constants
local ADDON_NAME = "TitanVPCap";
local VALOR_CURRENCY_ID = 396;
local JUSTICE_CURRENCY_ID = 395;
local ELDER_CHARM_CURRENCY_ID = 697;
local LESSER_CHARM_CURRENCY_ID = 738;

local WHITE_BRIGHT_COLOR = "|cffFFFFFF";
local YELLOW_NEON_COLOR = "|cffFFFF00";
local RED_NEON_COLOR = "|cffFF0000";
local ORANGE_NEON_COLOR = "|cffEE6622";
local GREEN_NEON_COLOR = "|cff00FF00";
local END_COLOR = FONT_COLOR_CODE_CLOSE;
local IVORY_COLOR = "|cFFFF00FF";


local CLASS_COLOR = {
	DEATHKNIGHT = "|cFFC41F3B",
	DRUID = "|cFFFF7D0A",
	HUNTER = "|cFFABD473",
	MAGE = "|cFF69CCF0",
	MONK = "|cFF00FF96",
	PALADIN = "|cFFF58CBA",
	PRIEST = "|cFFFFFFFF",
	ROGUE = "|cFFFFF569",
	SHAMAN = "|cFF0070DE",
	WARLOCK = "|cFF9482C9",
	WARRIOR = "|cFFC79C6E"
}

local L = LibStub("AceLocale-3.0"):GetLocale(ADDON_NAME, true);

-- Start Titan VP Cap Progress
VPCap = LibStub("AceAddon-3.0"):NewAddon(ADDON_NAME, "AceEvent-3.0", "AceConsole-3.0", "AceTimer-3.0");
VPCap.version = GetAddOnMetadata(ADDON_NAME, "Version");
VPCap.category = GetAddOnMetadata(ADDON_NAME, "X-Category");
VPCap.versionMajor = 1;    -- Used for DB versioning

-- Load Libs
VPCap.aceConfig = LibStub:GetLibrary("AceConfig-3.0");
VPCap.aceConfigDialog = LibStub:GetLibrary("AceConfigDialog-3.0");
VPCap.ldb = LibStub:GetLibrary("LibDataBroker-1.1");
VPCap.ldbIcon = LibStub:GetLibrary("LibDBIcon-1.0");
VPCap.callback = LibStub("CallbackHandler-1.0"):New(VPCap);

VPCap.currentPlayer = UnitName('player');	-- name of the logged-in character.
VPCap.currentRealm = GetRealmName();		-- current Realm we're on.
VPCap.menu = false;       -- Menu frame
VPCap.menuItems = {       -- Table for the dropdown menu
    top = {},
    middle = {},
    bottom = {},
};
VPCap.periodMaxVP = -1;
VPCap.maxJP = -1;
VPCap.maxVP = -1;


-- OnLoad
function VPCap:OnInitialize()

    -- Make sure everything is ok with the db
    if not TitanVPCap_Storage or type(TitanVPCap_Storage) ~= 'table' then TitanVPCap_Storage = {}; end
    
    -- Tell the player we are being loaded
	self:Print(format(L.core.load, self.version));
        
    -- Load settings
    self.db = LibStub("AceDB-3.0"):New("TitanVPCap_Settings", VPCap_Defaults, true);
	self:UpdateSettings();
	
    -- Set Up LDB
    local ldbObj = {
        type = "data source",
        icon = "Interface\\AddOns\\TitanVPCap\\TitanVPCap.tga",
        label = L.core.buttonLabel,
        text = 'n/a',
        category = self.category,
        version = self.version,
        OnClick = function(...) VPCap:ShowOptions(); end, -- update the database.
        OnTooltipShow = function(tt)
                            VPCap:SetTooltipText(tt);
						end,
    };
    
    -- Set back to a launcher if text is off
	if not self:GetLDB() then
		ldbObj.type = 'launcher';
		ldbObj.text = nil;
	end
    
    -- Start LDB
	self.ldb = self.ldb:NewDataObject(L.core.name, ldbObj);
	self.ldbUpdated = 0;
	self.ldbLabel = '';
    
    -- Start the minimap icon
	self.ldbIcon:Register(L.core.name, self.ldb, self.db.global.minimap);
    
    -- Register Options
	--VPCap_Options.args.purge.desc = format(L.core.options.purgeDesc, self:GetPurge() / 24);
	self.aceConfig:RegisterOptionsTable(L.core.name, VPCap_Options, {"VPCap", "TitanVPCap"});
	self.aceConfigDialog:AddToBlizOptions(L.core.name);

    
    -- Events
	self:RegisterEvent("MERCHANT_CLOSED", function() VPCap:UpdateLDB(true); end);
	self:RegisterEvent("PLAYER_ENTERING_WORLD", function() VPCap:UpdateLDB(true); end);
	self:RegisterEvent("UPDATE_INSTANCE_INFO", function() VPCap:UpdateLDB(true); end);
	self:RegisterEvent("PLAYER_LEAVING_WORLD", function() VPCap:UpdateLDB(true); end);
	self:RegisterEvent("RAID_INSTANCE_WELCOME", function() VPCap:UpdateLDB(true); end);
	self:RegisterEvent("LFG_UPDATE_RANDOM_INFO", function() VPCap:UpdateLDB(true); end);
	self:RegisterEvent("LFG_COMPLETION_REWARD", function() VPCap:UpdateLDB(true); end);
	self:RegisterEvent("PLAYER_LEAVE_COMBAT", function() VPCap:UpdateLDB(true); end);
    self:RegisterEvent("CURRENCY_DISPLAY_UPDATE", function() VPCap:UpdateLDB(true); end);

    -- Add to Paperdoll
    table.insert(PAPERDOLL_STATCATEGORIES["GENERAL"].stats, L.core.PDVPText);
	table.insert(PAPERDOLL_STATCATEGORIES["GENERAL"].stats, L.core.PDVPCapText);
	table.insert(PAPERDOLL_STATCATEGORIES["GENERAL"].stats, L.core.PDJPText);
	
	if self:GetVPPaperdoll() then
		PAPERDOLL_STATINFO[L.core.PDVPText] = { updateFunc = function(...) VPCap:UpdateVPPaperDollFrame(...); end };
	else
		PAPERDOLL_STATINFO[L.core.PDVPText] = nil;
	end

	if self:GetVPCapPaperdoll() then
		PAPERDOLL_STATINFO[L.core.PDVPCapText] = { updateFunc = function(...) VPCap:UpdateVPCapPaperDollFrame(...); end };
	else
		PAPERDOLL_STATINFO[L.core.PDVPCapText] = nil;
	end

	if self:GetJPPaperdoll() then
		PAPERDOLL_STATINFO[L.core.PDJPText] = { updateFunc = function(...) VPCap:UpdateJPPaperDollFrame(...); end };
	else
		PAPERDOLL_STATINFO[L.core.PDJPText] = nil;
	end
	
    -- Build the menu
    self:MenuInitialize();
    
    -- Get working on a score for the player
    self:UpdateLDB(); -- This may cause excesive loading time...
    
end

function VPCap:SetTooltipText(tt)
    -- Tooltip title
	local val = format(L.core.menuText, format(L.core.versionText, self.version));
	
	tt:SetText(val, HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);

	-- calculate the time remaining.
	local sec = time();
	local dtDate = sec - (sec % 86400) - 10*60*60;
	
	if sec%86400 < 10*60*60 then
	  dtDate = dtDate + 86400
	end
	
	local dtPrev = dtDate - (((date("%w", dtDate) - self.db.global.reset_dayofweek) % 7) ) * 86400 + (self.db.global.reset_hour * 60 * 60);
	if dtPrev > sec then
	  dtPrev = dtPrev - (7 * 24 * 60 * 60);
	end
	local dtNext = dtPrev + 7 * 24 * 60 * 60;
 	local timeRemaining = dtNext - sec;

	tt:AddLine("  " .. format(L.core.cooldownInfoText, WHITE_BRIGHT_COLOR .. self:GetRemainingTimeText(timeRemaining) .. END_COLOR));
	
	if self:GetDebug() then
		tt:AddLine("  Now: " .. WHITE_BRIGHT_COLOR  .. date("%d-%b-%Y %I:%M %p", time()) .. "   " .. time() .. END_COLOR);
		--tt:AddLine("  Last Reset: " .. WHITE_BRIGHT_COLOR  .. date("%d-%b-%Y %I:%M %p", dtPrev) .. "   " .. dtPrev .. END_COLOR);
		--tt:AddLine("  Date: " .. WHITE_BRIGHT_COLOR  .. date("%d-%b-%Y %I:%M %p", dtDate) .. "   " .. dtDate .. END_COLOR);
		--tt:AddLine("  Time: " .. WHITE_BRIGHT_COLOR  .. date("%d-%b-%Y %I:%M %p", sec) .. "   " .. sec .. END_COLOR);
		--tt:AddLine("  Resets: " .. WHITE_BRIGHT_COLOR  .. date("%d-%b-%Y %I:%M %p", dtNext) .. "   " .. dtNext .. END_COLOR);
	end
	tt:AddLine("  " .. format(L.core.cooldownInfoLastText, WHITE_BRIGHT_COLOR  .. date("%d-%b-%Y %I:%M %p", dtPrev) .. END_COLOR));
	tt:AddLine("  " .. format(L.core.cooldownInfoNextText, WHITE_BRIGHT_COLOR  .. date("%d-%b-%Y %I:%M %p", dtNext) .. END_COLOR));	
	tt:AddLine(" ");
	
	if self:GetWorldBoss() then
		self:ShowBosses(tt);
	end
	
	self:LoadAltVPCap(tt);
	tt:AddLine(" ");
	
	self:LoadAltJPCap(tt);
	tt:AddLine(" ");

	if self:GetCharms() then
		self:LoadAltCharm(tt);
		tt:AddLine(" ");
	end
end

function VPCap:ShowBosses(tt)

	tt:AddLine(L.core.BossTitle);	

	local z,t={[32099]=L.core.BossShaOfAnger,[32098]=L.core.BossGalleon},GetQuestsCompleted();
	for c,v in pairs(z) do 
		if t[c] then 
			tt:AddDoubleLine(RED_NEON_COLOR .. v, RED_NEON_COLOR .. L.core.BossLooted .. END_COLOR);
		else 
			tt:AddDoubleLine(RED_NEON_COLOR .. v, GREEN_NEON_COLOR .. L.core.BossNotLooted .. END_COLOR); 
		end 
	end
	
	tt:AddLine(" ");

end

function VPCap:LoadAltVPCap(tt)

	local sortorder;

	tt:AddLine(L.core.PDVPText);
	
	sortorder = self:GetSortOrder();
	
	if sortorder == 1 then  -- Unsorted
	
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do
			self:ShowAltVPCap(toonname, alt, tt);
		end
	
	elseif sortorder == 2 then  -- Alphabetical
		for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
			self:ShowAltVPCap(toonname, alt, tt);
		end
	
	elseif sortorder == 3 then  -- Class
		for class in orderedPairs(CLASS_COLOR) do
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.class == class then
					self:ShowAltVPCap(toonname, alt, tt);
				end
			end
		end
	
	elseif sortorder == 4 then  -- Points
		tmp = {};
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do 
			if not table.contains(tmp, alt.periodEarnVP) then
				table.insert(tmp, alt.periodEarnVP);
			end
		end
		table.sort(tmp, function(a,b) return a>b end);	
		for i, v in ipairs(tmp) do 
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.periodEarnVP == v then
					self:ShowAltVPCap(toonname, alt, tt);
				end
			end
		end
		
	elseif sortorder == 5 then  -- iLevel
		tmp = {};
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do 
			if not table.contains(tmp, alt.iLvl) then
				table.insert(tmp, alt.iLvl);
			end
		end
		table.sort(tmp, function(a,b) return a>b end);	
		for i, v in ipairs(tmp) do 
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.iLvl == v then
					self:ShowAltVPCap(toonname, alt, tt);
				end
			end
		end

	end
	
end

function VPCap:LoadAltJPCap(tt)
	
	local sortorder;

	tt:AddLine(L.core.PDJPText);
	
	sortorder = self:GetSortOrder();
	
	if sortorder == 1 then  -- Unsorted
	
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do
			self:ShowAltJPCap(toonname, alt, tt);
		end
		
	elseif sortorder == 2 then  -- Alphabetical
		for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
			self:ShowAltJPCap(toonname, alt, tt);
		end
	
	elseif sortorder == 3 then  -- Class
		for class in orderedPairs(CLASS_COLOR) do
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.class == class then
					self:ShowAltJPCap(toonname, alt, tt);
				end
			end
		end
	
	elseif sortorder == 4 then  -- Points
		tmp = {};
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do 
			if not table.contains(tmp, alt.qtyJP) then
				table.insert(tmp, alt.qtyJP);
			end
		end
		table.sort(tmp, function(a,b) return a>b end);	
		for i, v in ipairs(tmp) do 
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.qtyJP == v then
					self:ShowAltJPCap(toonname, alt, tt);
				end
			end
		end
	
	elseif sortorder == 5 then  -- iLevel
		tmp = {};
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do 
			if not table.contains(tmp, alt.iLvl) then
				table.insert(tmp, alt.iLvl);
			end
		end
		table.sort(tmp, function(a,b) return a>b end);	
		for i, v in ipairs(tmp) do 
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.iLvl == v then
					self:ShowAltJPCap(toonname, alt, tt);
				end
			end
		end
	
	end
		
end

function VPCap:LoadAltCharm(tt)
	
	local sortorder;

	tt:AddLine(L.core.PDCharmText);
	
	sortorder = self:GetSortOrder();
	
	if sortorder == 1 then  -- Unsorted
	
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do
			self:ShowAltCharm(toonname, alt, tt);
		end
		
	elseif sortorder == 2 then  -- Alphabetical
		for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
			self:ShowAltCharm(toonname, alt, tt);
		end
	
	elseif sortorder == 3 then  -- Class
		for class in orderedPairs(CLASS_COLOR) do
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.class == class then
					self:ShowAltCharm(toonname, alt, tt);
				end
			end
		end
	
	elseif sortorder == 4 then  -- Points
		tmp = {};
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do 
			if not table.contains(tmp, alt.LCharm) then
				table.insert(tmp, alt.qtyLCharm);
			end
		end
		table.sort(tmp, function(a,b) return a>b end);	
		for i, v in ipairs(tmp) do 
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.qtyLCharm == v then
					self:ShowAltCharm(toonname, alt, tt);
				end
			end
		end
	
	elseif sortorder == 5 then  -- iLevel
		tmp = {};
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do 
			if not table.contains(tmp, alt.iLvl) then
				table.insert(tmp, alt.iLvl);
			end
		end
		table.sort(tmp, function(a,b) return a>b end);	
		for i, v in ipairs(tmp) do 
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.iLvl == v then
					self:ShowAltCharm(toonname, alt, tt);
				end
			end
		end
	
	end
		
end

function VPCap:ShowAltVPCap(toon, alt, tt)
	local toonColor = "";
	local progressColor = "";
	local qtyColor = "";
	local progressText = "";
	local value = 0;
	
	local sec = time();
	local dtDate = sec - (sec % 86400) - 10*60*60 ; --+ 86400;

	if sec%86400 < 10*60*60 then
	  dtDate = dtDate + 86400
	end
	
	local dtPrev = dtDate - (((date("%w", dtDate) - self.db.global.reset_dayofweek) % 7) ) * 86400 + (self.db.global.reset_hour * 60 * 60);
	if dtPrev > sec then
	  dtPrev = dtPrev - (7 * 24 * 60 * 60);
	end

	if alt.qtyVP > 0 or self:GetDebug() then

		if alt.qtyMaxVP == nil then
			alt.qtyMaxVP = self.db.global.VPStandardMax;
		elseif alt.qtyMaxVP <= 0 then
			alt.qtyMaxVP = self.db.global.VPStandardMax;
		end 

		if alt.periodMaxVP == nil then
			alt.periodMaxVP = self.db.global.VPStandardCapMax;
		elseif alt.periodMaxVP <= 0 then
			alt.periodMaxVP = self.db.global.VPStandardCapMax;
		end 

		self:Debug("VP Max Cap: " .. alt.qtyMaxVP);
	
		for a, b in pairs(CLASS_COLOR) do
			if alt.class == a then
				toonColor = b;
			end
		end

		if dtPrev < alt.savingTime then
			value = alt.periodEarnVP;
		else
			value = 0;
		end

		if value >= alt.periodMaxVP then 
			progressColor = RED_NEON_COLOR;
		elseif value > (alt.periodMaxVP - self.db.global.progress_orange) then 
			progressColor = ORANGE_NEON_COLOR;
		elseif value > (alt.periodMaxVP - self.db.global.progress_yellow) then 
			progressColor = YELLOW_NEON_COLOR;
		else 
			progressColor = GREEN_NEON_COLOR;
		end

		if alt.qtyVP >= alt.qtyMaxVP then 
			qtyColor = RED_NEON_COLOR;
		elseif alt.qtyVP > (alt.qtyMaxVP - self.db.global.progress_orange) then 
			qtyColor = ORANGE_NEON_COLOR;
		elseif alt.qtyVP > (alt.qtyMaxVP - self.db.global.progress_yellow) then 
			qtyColor = YELLOW_NEON_COLOR;
		else 
			qtyColor = GREEN_NEON_COLOR;
		end

		progressText = value .. "/" .. alt.periodMaxVP;

		if self:GetDebug() then
			tt:AddDoubleLine(" " .. toonColor .. toon .. END_COLOR .. " (" .. qtyColor .. 1234 .. END_COLOR .. ")", progressColor .. "1234/1234" .. END_COLOR);
		else
			tt:AddDoubleLine(" " .. toonColor .. toon .. END_COLOR .. " (" .. qtyColor .. alt.qtyVP .. END_COLOR .. ")", progressColor .. progressText .. END_COLOR);
		end
	end
end

function VPCap:ShowAltJPCap(toon, alt, tt)
	local toonColor = "";
	local qtyColor = "";
	local value = 0;
	
	if alt.qtyJP > 0 or self:GetDebug() then

		if alt.qtyMaxJP ==nil then
			alt.qtyMaxJP = self.db.global.JPStandardMax;
		elseif alt.qtyMaxJP <= 0 then
			alt.qtyMaxJP = self.db.global.JPStandardMax;
		end 
		
		if alt.iLvl == nil then
			alt.iLvl = 0;
		end
	
		if alt.iELvl == nil then
			alt.iELvl = 0;
		end		
		
		self:Debug("JP Max Cap: " .. alt.qtyMaxJP);
	
		for a, b in pairs(CLASS_COLOR) do
			if alt.class == a then
				toonColor = b;
			end
		end

		value = alt.qtyJP;

		if value >= alt.qtyMaxJP then 
			qtyColor = RED_NEON_COLOR;
		elseif value > (alt.qtyMaxJP - self.db.global.progress_orange) then 
			qtyColor = ORANGE_NEON_COLOR;
		elseif value > (alt.qtyMaxJP - self.db.global.progress_yellow) then 
			qtyColor = YELLOW_NEON_COLOR;
		else 
			qtyColor = GREEN_NEON_COLOR;
		end

		self:Debug("Toon: " .. toon);
		self:Debug("QtyJP: " .. alt.qtyJP);
		self:Debug("maxJP: " .. alt.qtyMaxJP);
		self:Debug("Orangeqty: " .. self.db.global.progress_orange);
		self:Debug("Yellowqty: " .. self.db.global.progress_yellow);
		
		if self:GetDebug() then
			tt:AddDoubleLine(" " .. toonColor .. toon .. END_COLOR .. " (" .. GREEN_NEON_COLOR .. "i356/i456" .. END_COLOR .. ")", qtyColor .. "1234" .. END_COLOR);
		else
			if self:GetiLevel() and alt.iLvl > 0 then
				tt:AddDoubleLine(" " .. toonColor .. toon .. END_COLOR .. " (" .. GREEN_NEON_COLOR .. "i" .. alt.iLvl .. END_COLOR .. ")", qtyColor .. alt.qtyJP .. END_COLOR);
			else
				tt:AddDoubleLine(" " .. toonColor .. toon .. END_COLOR, qtyColor .. alt.qtyJP .. END_COLOR);
			end
		end
	end
end

function VPCap:ShowAltCharm(toon, alt, tt)
	local toonColor = "";
	local qtyColor = "";
	local Evalue = 0;
	local Lvalue = 0;
	
	if alt.qtyECharm == nil then
		alt.qtyECharm = 0;
	end
	
	if alt.qtyLCharm == nil then
		alt.qtyLCharm = 0;
	end
	
	if alt.qtyECharm > 0 or alt.qtyLCharm > 0 or self:GetDebug() then

		for a, b in pairs(CLASS_COLOR) do
			if alt.class == a then
				toonColor = b;
			end
		end

		Evalue = alt.qtyECharm;
		Lvalue = alt.qtyLCharm;
		qtyColor = GREEN_NEON_COLOR;

		self:Debug("Toon: " .. toon);
		self:Debug("QtyECharm: " .. alt.qtyECharm);
		self:Debug("QtyLCharm: " .. alt.qtyLCharm);
		
		if self:GetDebug() then
			tt:AddDoubleLine(" " .. toonColor .. toon .. END_COLOR, qtyColor .. "34/10" .. END_COLOR);
		else
			tt:AddDoubleLine(" " .. toonColor .. toon .. END_COLOR, qtyColor .. Lvalue .. "/" .. Evalue .. END_COLOR);
		end
	end
end


-- Save the data to the database.
function VPCap:SaveData()

	local qtyVP, qtyMaxVP, wk_earnVP, wk_MaxVP
	local qtyJP, wk_earnJP, qtyMaxJP
	local qtyECharm, qtyLCharm
	local iLvl, iELvl
	local a, b, c, d, e, f
	
	self:Debug("Saving VP Cap...");
	self:Debug("Current Player " .. self.currentPlayer);
	self:Debug("Current Realm " .. self.currentRealm);
	
	if TitanVPCap_Storage == nil then
		TitanVPCap_Storage = {};
	end
	if TitanVPCap_Storage[self.currentRealm] == nil then
		TitanVPCap_Storage[self.currentRealm] = {};
	end	
	if TitanVPCap_Storage[self.currentRealm][self.currentPlayer] == nil then
		TitanVPCap_Storage[self.currentRealm][self.currentPlayer] = {};
		local playerClass, englishClass = UnitClass("player");
		TitanVPCap_Storage[self.currentRealm][self.currentPlayer].class = englishClass;
	end
	TitanVPCap_Storage[self.currentRealm][self.currentPlayer].savingTime = time();
		
	a, qtyVP, b, wk_earnVP, wk_MaxVP, qtyMaxVP, e = GetCurrencyInfo(VALOR_CURRENCY_ID);
	a, qtyJP, b, wk_earnJP, c, qtyMaxJP, e = GetCurrencyInfo(JUSTICE_CURRENCY_ID);
	a, qtyECharm, b, c, d, e, f = GetCurrencyInfo(ELDER_CHARM_CURRENCY_ID);
	a, qtyLCharm, b, c, d, e, f = GetCurrencyInfo(LESSER_CHARM_CURRENCY_ID);
	iLvl, iELvl = GetAverageItemLevel();
	
	wk_MaxVP = floor(wk_MaxVP/100);
	qtyMaxVP = floor(qtyMaxVP/100);
	qtyMaxJP = floor(qtyMaxJP/100);
	iLvl = floor(iLvl);
	iELvl = floor(iELvl);
	
	-- save data 
	-- New values that are recorded.
	TitanVPCap_Storage[self.currentRealm][self.currentPlayer].qtyVP = qtyVP;
	TitanVPCap_Storage[self.currentRealm][self.currentPlayer].qtyMaxVP = qtyMaxVP;
	TitanVPCap_Storage[self.currentRealm][self.currentPlayer].periodEarnVP = wk_earnVP;
	TitanVPCap_Storage[self.currentRealm][self.currentPlayer].periodMaxVP = wk_MaxVP;
	
	TitanVPCap_Storage[self.currentRealm][self.currentPlayer].qtyJP = qtyJP;
	TitanVPCap_Storage[self.currentRealm][self.currentPlayer].periodEarnJP = wk_earnJP; 
	TitanVPCap_Storage[self.currentRealm][self.currentPlayer].qtyMaxJP = qtyMaxJP;
	
	TitanVPCap_Storage[self.currentRealm][self.currentPlayer].qtyECharm = qtyECharm;
	TitanVPCap_Storage[self.currentRealm][self.currentPlayer].qtyLCharm = qtyLCharm;
	
	if iLvl ~= 0 then
		TitanVPCap_Storage[self.currentRealm][self.currentPlayer].iLvl = iLvl;
	end
	
	if iELvl ~= 0 then
		TitanVPCap_Storage[self.currentRealm][self.currentPlayer].iELvl = iELvl;
	end

end


-- Make sure the database is the latest version
function VPCap:UpdateSettings()
    
    if self.db.global.version == self.versionMajor then
        -- Same version
    end
    
    -- Update version information
    self.db.global.version = self.versionMajor;
end

-- Purge the Database
function VPCap:PurgeDB()

	TitanVPCap_Storage = nil;
	
	self:SaveData();

end

--[[

    Setters, Getter and Togglers

]]

-- Set
function VPCap:SetLabel(v) self.db.global.showLabel = v; self:UpdateLDB(); end
function VPCap:SetLDBlabel(v) self.db.global.ldbLabel = v; self:UpdateLDB(true); end
function VPCap:SetLDBTextColor(v) self.db.global.ldbTextColor = v; self:UpdateLDB(true); end
function VPCap:SetCharms(v) self.db.global.charms = v; end
function VPCap:SetiLevel(v) self.db.global.iLevel = v; end
function VPCap:SetSortOrder(v) self.db.global.sortOrder = v; end
function VPCap:SetWorldBoss(v) self.db.global.worldBoss = v; end

-- Get
function VPCap:GetMinimap() return not self.db.global.minimap.hide; end
function VPCap:GetVPPaperdoll() return self.db.global.cinfoVP; end
function VPCap:GetVPCapPaperdoll() return self.db.global.cinfoVPCap; end
function VPCap:GetJPPaperdoll() return self.db.global.cinfoJP; end
function VPCap:GetLabel() return self.db.global.showLabel; end
function VPCap:GetLDB() return self.db.global.ldbText; end
function VPCap:GetLDBTextColor() return self.db.global.ldbTextColor; end
function VPCap:GetLDBlabel() return self.db.global.ldbLabel; end
function VPCap:GetDebug() return self.db.char.debug; end
function VPCap:GetCharms() return self.db.global.charms; end
function VPCap:GetiLevel() return self.db.global.iLevel; end
function VPCap:GetSortOrder() return self.db.global.sortOrder; end
function VPCap:GetWorldBoss() return self.db.global.worldBoss; end

-- Toggle
function VPCap:ToggleMinimap() self:SetMinimap(not self:GetMinimap()); end
function VPCap:ToggleCharms() self:SetCharms(not self:GetCharms()); end
function VPCap:ToggleiLevel() self:SetiLevel(not self:GetiLevel()); end
function VPCap:ToggleWorldBoss() self:SetWorldBoss(not self:GetWorldBoss()); end
function VPCap:ToggleSortOrder() self:SetSortOrder(not self:GetSortOrder()); end
function VPCap:ToggleVPPaperdoll() self:SetVPPaperdoll(not self:GetVPPaperdoll()); end
function VPCap:ToggleVPCapPaperdoll() self:SetVPCapPaperdoll(not self:GetVPCapPaperdoll()); end
function VPCap:ToggleJPPaperdoll() self:SetJPPaperdoll(not self:GetJPPaperdoll()); end
function VPCap:ToggleLabel() self:SetLabel(not self:GetLabel()); end
function VPCap:ToggleLDBlabel() self:SetLDBlabel(not self:GetLDBlabel()); end
function VPCap:ToggleLDBTextColor() self:SetLDBTextColor(not self:GetLDBTextColor()); end

-- Advanced Gets
function VPCap:GetRemainingTimeText(seconds)
	local days = floor(seconds / 86400);
	seconds = seconds - days * 86400;
	local hours = floor(seconds / 3600);
	seconds = seconds - hours * 3600;
	local minutes = floor(seconds / 60);
	seconds = seconds - minutes * 60;
	local TIME_COLOR = "";
	local TIME_TEXT = "";
	local timeStr = "";		-- one space just to pad slightly vs being too close to raid/instance name string.
	
	-- compact view's colors.
	-- moved "Reset in" text so that it's now above lockout time.
	TIME_COLOR = WHITE_BRIGHT_COLOR ;
	TIME_TEXT = WHITE_BRIGHT_COLOR ;
	
	if days > 0 then
		timeStr = timeStr .. TIME_COLOR .. days .. TIME_TEXT .. " " .. L.core.cooldownInfoDay .. END_COLOR .. " ";
	end
	
	if hours > 0 then
		timeStr = timeStr .. TIME_COLOR .. hours .. TIME_TEXT .. " " .. L.core.cooldownInfoHr .. END_COLOR .. " ";
	end

	if minutes then 
		timeStr = timeStr .. TIME_COLOR .. minutes .. TIME_TEXT .. " " .. L.core.cooldownInfoMin .. END_COLOR .. " ";
	end


	return timeStr;
end

function VPCap:GetButtonText()
	local a = " ";
	local val;
	local earnVP = TitanVPCap_Storage[self.currentRealm][self.currentPlayer].periodEarnVP;

	if self:GetPeriodMaxVP() <= 0 then
		val = "N/A";
	else
		val = earnVP .. "/" .. self:GetPeriodMaxVP();
	end

	self:Debug("PeriodMaxVP = " .. self:GetPeriodMaxVP());
	self:Debug("Earnt VP = " .. earnVP);
	
	if earnVP >= self:GetPeriodMaxVP() then 
		a = RED_NEON_COLOR;
	elseif earnVP > (self:GetPeriodMaxVP() - self.db.global.progress_orange) then 
		a = ORANGE_NEON_COLOR;
	elseif earnVP > (self:GetPeriodMaxVP() - self.db.global.progress_yellow) then 
		a = YELLOW_NEON_COLOR;
	else
		a = GREEN_NEON_COLOR;
	end

	if a then
	  self:Debug("col at line 405 has a value");
	else
	  self:Debug("col does not have a value");
	end 
	
	if self:GetLDBTextColor() then
		return " " .. a .. val .. END_COLOR;
	else
		return " " .. val;
	end
	
end

function VPCap:GetMaxVP()
	local a, b, c, d
	
	if self.maxVP >= 0 then
		return self.maxVP;
	else
		a, b, c, d, self.periodMaxVP, self.maxVP = GetCurrencyInfo(VALOR_CURRENCY_ID);
		self.periodMaxVP = floor(self.periodMaxVP / 100);
		self.maxVP = floor(self.maxVP/100);
		return self.maxVP;
	end

end

function VPCap:GetMaxJP()
	local a, b, c, d, e
	
	if self.maxJP >= 0 then
		return self.maxJP;
	else
		a, b, c, d, e, self.maxJP = GetCurrencyInfo(JUSTICE_CURRENCY_ID);
		self.maxJP = floor(self.maxJP/100);
		return self.maxJP;
	end

end

function VPCap:GetPeriodMaxVP()
	local a, b, c, d
	
	if self.periodMaxVP > 0 then
		return self.periodMaxVP;
	else
		a, b, c, d, self.periodMaxVP, self.maxVP = GetCurrencyInfo(VALOR_CURRENCY_ID);
		self.periodMaxVP = floor(self.periodMaxVP / 100);
		self.maxVP = floor(self.maxVP/100);
		
		return self.periodMaxVP;
	end

end

function VPCap:GetResetHour() 
	if self.db.global.reset_hour < 0 or self.db.global.reset_hour > 23 then
		self.db.global.reset_hour = VPCap_Defaults.global.reset_hour;
	end
	
	return self.db.global.reset_hour; 
	
end

function VPCap:GetResetDayOfWeek() 
	if self.db.global.reset_dayofweek < 1 or self.db.global.reset_dayofweek > 7 then
		self.db.global.reset_dayofweek = VPCap_Defaults.global.reset_dayofweek;
	end
	
	return self.db.global.reset_dayofweek; 

end


-- Advanced sets
function VPCap:SetMinimap(v) 
    self.db.global.minimap.hide = not v;
	
	if not v then
		self.ldbIcon:Hide(L.core.name);
	else
		self.ldbIcon:Show(L.core.name);
	end
end

function VPCap:SetResetHour(v) 
	if v < 0 or v > 23 then
		self.db.global.reset_hour = VPCap_Defaults.global.reset_hour;
	else
		self.db.global.reset_hour = v; 
	end
	
end

function VPCap:SetResetDayOfWeek(v) 
	if v < 1 or v > 7 then
		self.db.global.reset_dayofweek = VPCap_Defaults.global.reset_dayofweek;
	else
		self.db.global.reset_dayofweek = v;
	end 

end

function VPCap:SetDebug(v) 

	self.db.char.debug = v; 
	
	if v then 
		self:Print("Entering debug mode");
	else 
		self:Print("Leaving debug mode");
	end

end

function VPCap:SetLDB(v)
	self.db.global.ldbText = v;
	
	if v then
		self.ldb.type = 'data source';
	else
		self.ldb.type = 'launcher';
		self.ldb.text = nil;
	end
	
	self:UpdateLDB(true);
end

function VPCap:SetVPPaperdoll(v)
	self.db.global.cinfoVP = v;
	
	if v then
		PAPERDOLL_STATINFO[L.core.PDVPText] = { updateFunc = function(...) VPCap:UpdateVPPaperDollFrame(...); end };
	else
		PAPERDOLL_STATINFO[L.core.PDVPText] = nil;
	end
end

function VPCap:SetVPCapPaperdoll(v)
	self.db.global.cinfoVPCap = v;
	
	if v then
		PAPERDOLL_STATINFO[L.core.PDVPCapText] = { updateFunc = function(...) VPCap:UpdateVPCapPaperDollFrame(...); end };
	else
		PAPERDOLL_STATINFO[L.core.PDVPCapText] = nil;
	end
end

function VPCap:SetJPPaperdoll(v)
	self.db.global.cinfoJP = v;
	
	if v then
		PAPERDOLL_STATINFO[L.core.PDJPText] = { updateFunc = function(...) VPCap:UpdateJPPaperDollFrame(...); end };
	else
		PAPERDOLL_STATINFO[L.core.PDJPText] = nil;
	end
end


-- Advanced Updates
function VPCap:UpdateLDB(force, auto)
	
	self:SaveData();
	
	if self:GetLDB() then
		local label = UnitName('player');
		
		-- Do we really need to update LDB?
		if force or label ~= self.ldbLabel then
            
            local text = self:GetButtonText();
			self:UpdateLDBText(label, text)

		end
	else
		self.ldb.type = 'launcher';
		self.ldb.text = nil;
	end
end

function VPCap:UpdateLDBText(label, text)
    
	if not self:GetLDB() then return false; end
    
    -- Add the label
    if self:GetLDBlabel() then
        text = label..": "..text;
    end
    
    self.ldb.text = text;
    self.ldbUpdated = time();
    self.ldbLabel = label;
end

function VPCap:UpdateVPPaperDollFrame(statFrame, unit)
    
	local formatted;
	
	self:SaveData();
	
	PaperDollFrame_SetLabelAndText(statFrame, L.core.PDVPText, TitanVPCap_Storage[self.currentRealm][self.currentPlayer].qtyVP, false);
    statFrame.tooltip = HIGHLIGHT_FONT_COLOR_CODE..L.core.PDVPText..FONT_COLOR_CODE_CLOSE;
    if self:GetMaxVP() > 0 then
	  formatted = L.core.PDVPTooltip .. "\n" .. format(L.core.PDVPTooltipMax, self:GetMaxVP());
	else
	  formatted = L.core.PDVPTooltip;
    end
    statFrame.tooltip2 = formatted;
	
    statFrame:Show();
end

function VPCap:UpdateVPCapPaperDollFrame(statFrame, unit)
    
	local formatted;
	
	if self:GetPeriodMaxVP() > 0 then
	  formatted = TitanVPCap_Storage[self.currentRealm][self.currentPlayer].periodEarnVP .. "/" .. self:GetPeriodMaxVP();
	else
	  formatted = TitanVPCap_Storage[self.currentRealm][self.currentPlayer].periodEarnVP;
	end
	
    PaperDollFrame_SetLabelAndText(statFrame, L.core.PDVPCapText, formatted, false);
    statFrame.tooltip = HIGHLIGHT_FONT_COLOR_CODE..L.core.PDVPCapText..FONT_COLOR_CODE_CLOSE;
    if self:GetPeriodMaxVP() > 0 then
	  formatted = L.core.PDVPCapTooltip .. "\n" .. format(L.core.PDVPCapTooltipMax, self:GetPeriodMaxVP());
	else
	  formatted = L.core.PDVPCapTooltip;
    end
    statFrame.tooltip2 = formatted;
    
    statFrame:Show();
end

function VPCap:UpdateJPPaperDollFrame(statFrame, unit)
    	
    PaperDollFrame_SetLabelAndText(statFrame, L.core.PDJPText, TitanVPCap_Storage[self.currentRealm][self.currentPlayer].qtyJP, false);
    statFrame.tooltip = HIGHLIGHT_FONT_COLOR_CODE..L.core.PDJPText..FONT_COLOR_CODE_CLOSE;
    if self:GetMaxJP() > 0 then
	  formatted = L.core.PDJPTooltip .. "\n" .. format(L.core.PDJPTooltipMax, self:GetMaxJP());
	else
	  formatted = L.core.PDJPTooltip;
    end
    statFrame.tooltip2 = formatted;
	
    statFrame:Show();
end


-- Debug function
function VPCap:Debug(...)
	if self:GetDebug() then
		print('|cFFFF0000TitanVPCap Debug:|r ', ...);
	end
end

-- Open the options window
function VPCap:ShowOptions()
    -- Not using this until a paramiter is added to open expanded
    -- InterfaceOptionsFrame_OpenToCategory(L.core.name);
    
	self:SaveData();
	
    for i,element in pairs(INTERFACEOPTIONS_ADDONCATEGORIES) do 
        if element.name == L.core.name then
            if not InterfaceOptionsFrame:IsShown() then
                InterfaceOptionsFrame_Show();
                InterfaceOptionsFrameTab2:Click();
            end

            for i,button in pairs(InterfaceOptionsFrameAddOns.buttons) do
                if button.element == element then
                    InterfaceOptionsListButton_OnClick(button);
                    
                    -- Expand
                    if element.hasChildren and element.collapsed then
                        OptionsListButtonToggle_OnClick(button.toggle);
                    end
                
                -- Hide anything else that is open
                elseif button.element and type(button.element.collapsed) == 'boolean' and not button.element.collapsed then
                    OptionsListButtonToggle_OnClick(button.toggle);
                end
            end
        end
    end
end



--[[

local GRAY_DARK_COLOR = "|cff808080";
local GRAY_VPCapVER_COLOR = "|cffDBD5CA";
local YELLOW_BRIGHTPALE_COLOR = "|cffEBFC07";
local BLUE_NEON_COLOR = "|cff08FAFA";
local GREEN_NEON_DIMMED_COLOR = "|cff4BC143";
local GREEN_NEON_DARK_COLOR = "|cff449F3E";
local GREEN_DARK_COLOR = "|cff3A8048";
local GREEN_SLATE_COLOR = "|cff7A9079";
local RED_NEON_PURPLEPINK_COLOR = "|cffFC046F";
local ORANGE_DIM_COLOR = "|cffBD8F3E";
local ORANGE_LESSDIM_COLOR = "|cffDAAC41";





local color_defaultAltName = WHITE_BRIGHT_COLOR;
local color_defaultTimeText = WHITE_BRIGHT_COLOR;
local color_defaultResetIn = WHITE_BRIGHT_COLOR;
local color_defaultDashLine = END_COLOR;

]]